# DO NOT EDIT THIS FILE DIRECTLY (unless you're in the `infrastructure-charts` repo)
#
# Periodically run `make update-helm.mk` to update this file with the latest version
#
# This helper makefile is designed to make it easier for Developers to install
# local helm charts in their application repos (by following some conventions.)  Using this
# file is not required, if you prefer to use some other build rigging (maven, etc.)
# This makefile does not handle "building" the charts, simply installing/uninstalling.
#
# Full documentation is available at https://confluence.eng.vmware.com/display/MCS2/Create+Dev+Environment#CreateDevEnvironment-B.CreatingENV
#
# Summary Usage:
# * Copy this file to your top-level repo path
# * Add 'include helm.mk' at the END of your Makefile
# * Define INFRA_CHARTS as a list of infrastructure chart(s) triples ('__' delimited)
#   release-name__chart_name__chart_version
# * Define APP_CHARTS as a list of application chart(s) triples ('__' delimited)
# * Set RELEASED_CHARTS to a non-empty string to force installing from published harbor repos
#   By default, it will install from the local build path (MAKE SURE TO BUILD BEFORE INSTALL!)
#


ENV?=$(USER)
ENV_DIR ?= $(HOME)/.cbc/envs/$(ENV)
ENV_DEFINITION = $(ENV_DIR)/env-definitions.yaml
INFRA_NAMESPACE = $(shell cat $(ENV_DEFINITION) | grep infraNamespace | cut -d: -f 2 | xargs)
APP_NAMESPACE = $(shell cat $(ENV_DEFINITION) | grep appsNamespace | cut -d: -f 2 | xargs)

# Use harbor-sbu-dev for staging builds, harbor-sbu for prod builds
# To be removed
CHART_PUBLISH_REPO?=harbor-sbu-dev
CHART_PUBLISH_REGISTRY?=oci://$CB_HELM_REGISTRY_DEV/sbu-dev/charts
INFRA_RELEASES:=$(foreach tgt,$(INFRA_CHARTS),$(word 1,$(subst __, ,$(tgt))))
APP_RELEASES:=$(foreach tgt,$(APP_CHARTS),$(word 1,$(subst __, ,$(tgt))))
HELM_TIMEOUT?=20m
HELM_DEBUG?=false
LOCAL_HELM_DIR?=./builds
HELM_EXPERIMENTAL_OCI?=1

TEMPLATE_APP_TARGETS=$(foreach tgt,$(APP_RELEASES),$(tgt)__app_template)
TEMPLATE_INFRA_TARGETS=$(foreach tgt,$(INFRA_RELEASES),$(tgt)__infra_template)

# Build up the pattern rules based on the name of the release names
ifdef RELEASED_CHARTS
INSTALL_APP_TARGETS=$(foreach tgt,$(APP_RELEASES),$(tgt)__app_install)
INSTALL_INFRA_TARGETS=$(foreach tgt,$(INFRA_RELEASES),$(tgt)__infra_install)
else
INSTALL_APP_TARGETS=$(foreach tgt,$(APP_RELEASES),$(tgt)__app_local_install)
INSTALL_INFRA_TARGETS=$(foreach tgt,$(INFRA_RELEASES),$(tgt)__infra_local_install)
endif

UNINSTALL_APP_TARGETS=$(foreach tgt,$(APP_RELEASES),$(tgt)__app_uninstall)
UNINSTALL_INFRA_TARGETS=$(foreach tgt,$(INFRA_RELEASES),$(tgt)__infra_uninstall)

PREVIEW_APP_TARGETS=$(foreach tgt,$(APP_RELEASES),$(tgt)__app_preview)
PREVIEW_INFRA_TARGETS=$(foreach tgt,$(INFRA_RELEASES),$(tgt)__infra_preview)

ifeq ("${HELM_DEBUG}","true")
	HELM_DEBUG_OPT = --dry-run --debug
else
	HELM_DEBUG_OPT =
endif


export ENV
export ENV_DIR
export ENV_DEFINITION
export INFRA_NAMESPACE
export APP_NAMESPACE
export HELM_EXPERIMENTAL_OCI


# Helper functions
find_chart = $(foreach x,$(1),$(filter $(2),$x))
extract_chart_name = $(word 2,$(subst __, ,$(1)))
extract_chart_version = $(word 3,$(subst __, ,$(1)))
create_remote_chart_name = $(CHART_PUBLISH_REPO)/$(call extract_chart_name,$(call find_chart,$(1),$(2)))
create_local_chart_path = $(LOCAL_HELM_DIR)/$(call extract_chart_name,$(call find_chart,$(1),$(2)))-$(call extract_chart_version,$(call find_chart,$(1),$(2))).tgz
find_extract_chart_version = $(call extract_chart_version,$(call find_chart,$(1),$(2)))

.PHONY: uninstall install uninstall-infra install-infra uninstall-app install-app
install: install-infra install-app

template: template-infra template-app

template-infra: $(TEMPLATE_INFRA_TARGETS)

template-app: $(TEMPLATE_APP_TARGETS)


install-infra: $(INSTALL_INFRA_TARGETS)

install-app: $(INSTALL_APP_TARGETS)

uninstall:  uninstall-app uninstall-infra

uninstall-infra: $(UNINSTALL_INFRA_TARGETS)

uninstall-app: $(UNINSTALL_APP_TARGETS)

.PHONY: preview preview-infra preview-app
preview: preview-infra preview-app

preview-infra: $(PREVIEW_INFRA_TARGETS)

preview-app: $(PREVIEW_APP_TARGETS)


%__infra_template: $(ENV_DEFINITION)
	@echo "LOCAL: Templating locally built $*"
	helm --namespace $(INFRA_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		$(HELM_DEBUG_OPT) template \
		$(call create_local_chart_path,$(INFRA_CHARTS),$*%)

%__app_template: $(ENV_DEFINITION)
	@echo "LOCAL: Templating locally built $*"
	helm --namespace $(APP_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		$(HELM_DEBUG_OPT) template \
		$(call create_local_chart_path,$(APP_CHARTS),$*%)

# TODO DRY these out a bit more... (or switch to the manifest tooling)
%__infra_install: $(ENV_DEFINITION)
	@echo "HARBOR: Installing/Upgrading chart $* from harbor to $(INFRA_NAMESPACE)"
	helm --namespace $(INFRA_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		--set global.env=$(ENV) \
		--version $(call find_extract_chart_version,$(INFRA_CHARTS),$*%) \
		$(HELM_DEBUG_OPT) upgrade --install \
		--wait --timeout $(HELM_TIMEOUT) \
		$* \
		$(call create_remote_chart_name,$(INFRA_CHARTS),$*%)

%__app_install: $(ENV_DEFINITION)
	@echo "HARBOR: Installing/Upgrading chart $* from harbor to $(APP_NAMESPACE)"
	helm --namespace $(APP_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		--set global.env=$(ENV) \
		--version $(call find_extract_chart_version,$(APP_CHARTS),$*%) \
		$(HELM_DEBUG_OPT) upgrade --install \
		--wait --timeout $(HELM_TIMEOUT) \
		$* \
		$(call create_remote_chart_name,$(APP_CHARTS),$*%)

%__infra_local_install: $(ENV_DEFINITION)
	@echo "LOCAL: Installing/Upgrading locally built $* to $(INFRA_NAMESPACE)"
	helm --namespace $(INFRA_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		--set global.env=$(ENV) \
		$(HELM_DEBUG_OPT) upgrade --install \
		--wait --timeout $(HELM_TIMEOUT) \
		$* \
		$(call create_local_chart_path,$(INFRA_CHARTS),$*%)

%__app_local_install: $(ENV_DEFINITION)
	@echo "LOCAL: Installing/Upgrading locally built $* to $(APP_NAMESPACE)"
	helm --namespace $(APP_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		--set global.env=$(ENV) \
		$(HELM_DEBUG_OPT) upgrade --install \
		--wait --timeout $(HELM_TIMEOUT) \
		$* \
		$(call create_local_chart_path,$(APP_CHARTS),$*%)

%__infra_uninstall:
	helm --namespace $(INFRA_NAMESPACE) \
		--timeout $(HELM_TIMEOUT) \
		$(HELM_DEBUG_OPT) uninstall \
		$*

%__app_uninstall:
	helm --namespace $(APP_NAMESPACE) \
		--timeout $(HELM_TIMEOUT) \
		$(HELM_DEBUG_OPT) uninstall \
		$*

%__infra_preview: $(ENV_DEFINITION)
	helm --namespace $(INFRA_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		--set global.env=$(ENV) \
		--version $(call find_extract_chart_version,$(INFRA_CHARTS),$*%) \
		template --debug \
		$* \
		$(call create_local_chart_path,$(INFRA_CHARTS),$*%)

%__app_preview: $(ENV_DEFINITION)
	helm --namespace $(APP_NAMESPACE) \
		--values $(ENV_DIR)/env-definitions.yaml \
		--set global.env=$(ENV) \
		--version $(call find_extract_chart_version,$(APP_CHARTS),$*%) \
		template --debug \
		$* \
		$(call create_local_chart_path,$(APP_CHARTS),$*%)


.PHONY: update-helm.mk
update-helm.mk:
	curl https://gitlab.bit9.local/mcs2/infrastructure-charts/-/raw/master/mks/helm.mk?inline=false -o helm.mk


#Help the user figure out how to use the helm registry
_verify_helm_login:
	@if [ -f /Users/$${USER}/Library/Preferences/helm/registry.json ]; then \
		if ! grep "harbor-repo.vmware.com" /Users/$${USER}/Library/Preferences/helm/registry.json &> /dev/null ; then \
			echo "ERROR: please call the following command to login registry" ; \
			echo "helm registry login harbor-repo.vmware.com -u $${USER}"; \
			exit 1; \
		fi; \
	fi

_verify_helm_patch:
	echo "Install helm-dependency-update-oci"
	curl -s --fail https://gitlab.bit9.local/mcs2/devops/-/raw/master/common-packages/helm-dependency-update-oci/bin/helm-dependency-update-oci -o helm-dependency-update-oci && chmod +x helm-dependency-update-oci


$(ENV_DEFINITION):
	@if [ ! -f "$(ENV_DEFINITION)" ] ; then \
		echo "ERROR: '$(ENV_DEFINITION)' does not exists!" ; \
		echo "Please refer to this link to create your ENV https://confluence.eng.vmware.com/display/MCS2/Create+Dev+Environment#CreateDevEnvironment-CreatingCluster; \
		echo "Please use `kubectl config use-context xxx;  make create-env` to create $(ENV_DIR) "; \
		exit 1; \
	fi



.PHONY: help-helm.mk
help-helm.mk:
	@echo "install-infra         : helm upgrade/install chart (HELM_DEBUG=false)"
	@echo "install-app           : helm update/install app chart (HELM_DEBUG=false)"
	@echo "install               : helm upgrade/install infra and apps (HELM_DEBUG=false)"
	@echo "uninstall-infra       : helm uninstall chart (HELM_DEBUG=false)"
	@echo "uninstall-app         : helm uninstall app chart (HELM_DEBUG=false)"
	@echo "uninstall             : helm uninstall infra and apps (HELM_DEBUG=false)"
	@echo "template-infra        : helm template infra (HELM_DEBUG=false)"
	@echo "template-app          : helm template apps (HELM_DEBUG=false)"
	@echo "template              : helm template infra and apps (HELM_DEBUG=false)"
	@echo "update-helm.mk        : fetch helm.mk from https://gitlab.bit9.local/mcs2/infrastructure-charts"
	@echo "help-helm.mk          : display this menu"
	@echo ""

