import logging
import random
import sys
import time

from prometheus_client import Counter, Histogram
from prometheus_client import start_http_server, multiprocess, CollectorRegistry

logging.basicConfig(stream=sys.stdout, level=logging.INFO)


class MetricsClient:
    """
    A streaming metric client that instantiates a Prometheus Collector and provides metrics like count, latency.
    """

    def __init__(
            self,
            metric_prefix,
            metrics_port,
    ):
        """
        Initializes the Metrics Client
        :param metric_prefix: Prefix of all metrics sent to Wavefront
        :param metrics_port: Port on which metrics are sent
        :param app: The flask app for which we want to return
        """

        self.metric_prefix = metric_prefix
        self.metrics_port = int(metrics_port)
        self.content_type = str("text/plain; version=0.0.4; charset=utf-8")

        registry = CollectorRegistry()

        self.org_count = Counter(
            f"{self.metric_prefix}_org",
            "Org processed Count",
            ["app_name", "job_type", "env", "id"],
            registry=registry,
        )

        self.org_err = Counter(
            f"{self.metric_prefix}_orgError",
            "Org error Count",
            ["app_name", "job_type"],
            registry=registry,
        )

        self.start_metrics_server(registry)

    def start_metrics_server(self, registry):
        """
        lazy_apps=true uwsgi mode starts this flask app on each worker process and attempts to start the metric server
         multiple times. To prevent that we are first checking if port is not in use then only start the metrics server.
        The thread here will keep checking every 30 seconds if port is free. If the worker whose thread started the
        server dies, then other worker will restart the metrics server.
        """
        try:
            import socket
            import threading

            def is_port_in_use(port):
                """
                Checks if a port is already in use
                """
                with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                    return s.connect_ex(("localhost", port)) == 0

            def check_port_status_thread():
                """
                Thread function that periodically checks if the metrics_port is in use
                """
                while True:
                    if is_port_in_use(self.metrics_port):
                        logging.debug(f"Metrics Port {self.metrics_port} is in use")
                    else:
                        start_http_server(self.metrics_port, registry=registry)
                        logging.info(f"Metrics Port {self.metrics_port} is free")
                    time.sleep(
                        random.randint(15, 30)
                    )  # Wait for 15 - 30 seconds before checking again

            t = threading.Thread(target=check_port_status_thread)
            t.daemon = True
            t.start()

        except Exception as e:
            logging.info(e)
