import unittest
from csv import DictReader
from io import StringIO

import boto3
from moto import mock_s3

from src.daily_report_job import get_yesterdays_date
from src.monthly_report_job import get_last_month
from src.utils.reporter import MetricsClient
from src.utils.utils import convert_statistics_csv, org_data_to_S3


def process_csv_from_s3(bucket, key, s3_client):
    results = s3_client.get_object(Bucket=bucket, Key=key)['Body']
    return DictReader(StringIO(results.read().decode('utf-8')), delimiter=",")


class MdrReportingQueryTest(unittest.TestCase):

    @mock_s3
    def test_daily_query(self):
        # Setup mock S3
        s3 = boto3.client('s3', region_name='us-east-1')
        query_bucket = 'query-results'
        s3.create_bucket(Bucket=query_bucket)
        key = 'daily-query-result.csv'
        file_path = './tests/data/daily_query_result.csv'
        # Upload local query result to mocked s3 bucket
        with open(file_path, 'rb') as file:
            s3.put_object(Bucket=query_bucket, Key=key, Body=file)
        stat_results = process_csv_from_s3(query_bucket, key, s3)

        org_objects = {}

        # Process query result
        metric_client = MetricsClient('mdr_reporting_job', '9090')
        convert_statistics_csv(stat_results, org_objects, metric_client, 'daily')

        # Put categorized org alert data to s3
        daily_alert_bucket = 'daily-alert-bucket'
        s3.create_bucket(Bucket='daily-alert-bucket')
        date = get_yesterdays_date()
        org_data_to_S3(org_objects, bucket='daily-alert-bucket', file_name=date)


        # Additional assertions can be made here to verify the contents in S3
        response = s3.get_object(Bucket=daily_alert_bucket, Key=f'test03-9539693/{date}')
        data_in_s3 = response['Body'].read().decode('utf-8')
        assert data_in_s3 is not None

    @mock_s3
    def test_monthly_query(self):
        # Setup mock S3
        s3 = boto3.client('s3', region_name='us-east-1')
        query_bucket = 'query-results'
        key = 'monthly_query_result.csv'
        file_path = './tests/data/monthly_query_result.csv'
        s3.create_bucket(Bucket=query_bucket)
        # Upload local query result to mocked s3 bucket
        with open(file_path, 'rb') as file:
            s3.put_object(Bucket=query_bucket, Key=key, Body=file)

        results = process_csv_from_s3(query_bucket, key, s3)

        # Process query result
        org_objects = {}
        metric_client = MetricsClient('mdr_reporting_job', '9090')
        convert_statistics_csv(results, org_objects, metric_client, 'daily')

        # Put categorized org alert data to s3
        monthly_bucket = 'monthly-bucket'
        s3.create_bucket(Bucket=monthly_bucket)
        date = get_last_month()
        org_data_to_S3(org_objects, bucket=monthly_bucket, file_name=date)


        # Additional assertions can be made here to verify the contents in S3
        response = s3.get_object(Bucket=monthly_bucket, Key=f'test03-1/{date}')
        data_in_s3 = response['Body'].read().decode('utf-8')
        assert data_in_s3 is not None
