{{/*
Common labels
*/}}
{{- define "s3.labels" -}}
{{ include "s3-utils.labels" . }}
{{- end -}}

{{/*
Selector labels
*/}}
{{- define "s3.selectorLabels" -}}
{{ include "s3-utils.selectorLabels" . }}
{{- end -}}

{{/*
     S3 Bucket Name
*/}}
{{- define "s3.bucketName" -}}
{{ include "s3-utils.externalNamingPrefix" . }}
{{- end -}}

{{/*
    S3 Bucket Policy
*/}}
{{- define "s3.bucketPolicy" -}}
{
    "Version": "2012-10-17",
    "Id": "PutObjPolicy",
    "Statement": [
        {
            "Sid": "DenyNonSSLAccess",
            "Effect": "Deny",
            "Principal": "*",
            "Action": "s3:*",
            "Resource": "{{ .Values.global.awsArnPrefix | default "arn:aws" }}:s3:::{{- include "s3.bucketName" . }}/*",
            "Condition": {
                "Bool": {
                    "aws:SecureTransport": "false"
                }
            }
        }{{- if .Values.enableBucketOwnerFullControlAcl -}},
        {
            "Sid": "AllowIfBucketOwnerFullControlAclOnly",
            "Effect": "Deny",
            "Principal": {
                "AWS": "*"
            },
            "Action": "s3:PutObject",
            "Resource": "{{ .Values.global.awsArnPrefix | default "arn:aws" }}:s3:::{{- include "s3.bucketName" . }}/*",
            "Condition": {
                "StringNotEquals": {
                "s3:x-amz-acl": "bucket-owner-full-control"
                }
            }
        }
        {{- end -}}
        {{- if .Values.bucketPolicyStatements -}}
        {{ if contains "gov" .Values.global.awsRegion -}}
        {{- printf "\n\nThe bucketPolicyStatements is not supported in gov environments." | fail -}}
        {{- else -}},
        {{ tpl .Values.bucketPolicyStatements $ | indent 10 -}}
        {{- end -}}
        {{- end -}}
        {{- if .Values.cloudfront.enable -}},
        {{- include "s3.bucketPolicy.cloudfront" . | indent 8 -}}
        {{- end -}}
    ]
}
{{- end -}}

{{/*
    Cloudfront S3 Bucket Policy Statement
*/}}
{{ define "s3.bucketPolicy.cloudfront" }}
{
    "Sid": "S3TrustCloudfrontOAI",
    "Effect": "Allow",
    "Principal": {
        "AWS": "{{ required "You must specify the 'cloudfrontOriginAccessId' to use this bucket as a Cloudfront origin." .Values.cloudfront.cloudfrontOriginAccessId }}"
    },
    "Action": "s3:GetObject",
    "Resource": "{{ .Values.global.awsArnPrefix | default "arn:aws" }}:s3:::{{- include "s3.bucketName" . }}/*"
}
{{- end -}}

{{/*
    S3 Event Type Allowed List
*/}}
{{ define "s3.allowedEventTypes" }}
{{- printf "s3:ObjectCreated:*,s3:ObjectRemoved:*,s3:LifecycleExpiration:*" -}}
{{- end -}}

{{/*
    Verify S3 events type and form a comma separated string for terraform input
*/}}
{{- define "s3.eventTypes" -}}
{{- $allowedEventTypes := include "s3.allowedEventTypes" . }}
{{- $events := required "You must specify the list of 'eventTypes' for S3 Events" .Values.events.eventTypes }}
{{- range $events }}
{{- if not ( has . (splitList "," $allowedEventTypes) ) -}}
{{- printf "\nInvalid event in the list of eventTypes, The Allowed Events are \"s3:ObjectCreated:*,s3:ObjectRemoved:*,s3:LifecycleExpiration:*\"" | fail -}}
{{- end -}}
{{- end -}}
{{- join "," $events | quote }}
{{- end -}}

{{/*
    Ensure that only one event has been enabled and error out otherwise
*/}}
{{- define "s3.checkEvent" -}}
{{- $lambdaEnabled := .Values.events.lambdaEnable | default false -}}
{{- $sqsEnabled := .Values.events.sqsEnable | default false -}}
{{- $snsEnabled := .Values.events.snsEnable | default false -}}
{{- $triggers := list $lambdaEnabled $sqsEnabled $snsEnabled -}}
{{- $count := 0 -}}
{{- range $triggers }}
{{- if eq . true -}}
{{- $count = add $count 1 -}}
{{- end -}}
{{- end -}}
{{- if eq $count 0 -}}
{{- printf "\n When S3 events is enabled, please enable either one of the S3 Events Destinations: lambda, sqs or sns. Note: We currently do not support multiple destinations for S3 Events " | fail -}}
{{- end -}}
{{- if gt $count 1 -}}
{{- printf "\n Please enable either one of the S3 Events Destinations ONLY:lambda, sqs or sns. We currently do not support multiple destinations for S3 Events " | fail -}}
{{- end -}}
{{- end -}}

{{/*
    Allowed S3 Storage Class for Lifecycle Policy
*/}}
{{- define "s3.allowedStorageClasses" -}}
{{- printf "ONEZONE_IA,STANDARD_IA,INTELLIGENT_TIERING,GLACIER,DEEP_ARCHIVE" }}
{{- end -}}

{{/*
    Check if atleast one of the lifecycle properties has been configured
*/}}
{{- define "s3.checkLifecyclePropertyValid" -}}
{{- if not ( or  .expiration .transitions .noncurrentVersionExpiration .noncurrentVersionTransitions .abortIncompleteMultipartUploadDays ) -}}
{{- printf "\n When lifecycle.enable is set to true, Please enable atleast one of the lifecycle properties:expiration, transitions , noncurrentVersionExpiration, noncurrentVersionTransitions, abortIncompleteMultipartUploadDays" | fail -}}
{{- end -}}
{{- end -}}


{{/*
    Check if the S3 Lifecycle expiration config is valid
    - check if the number of days added as expiration days is more than 0
    - ensure that either days or expiration marker has been set for the expiration rule
*/}}
{{- define "s3.checkExpirationConfigValid" -}}
{{- $expirationDays := .days | default false -}}
{{- $expiredObjectDeleteMarker := .expiredObjectDeleteMarker | default false -}}
{{- if and $expirationDays $expiredObjectDeleteMarker -}}
{{ printf "\nYou must specify values for either days or expiredObjectDeleteMarker in S3 expiration lifecycle rule, not both" | fail }}
{{- end -}}
{{- if and (not $expirationDays) (not $expiredObjectDeleteMarker) -}}
{{ printf "\nYou must specify values for atleast one of the following expiration properties: days or expiredObjectDeleteMarker" | fail }}
{{- end -}}
{{- if $expirationDays -}}
{{- include "s3.checkExpirationDayValid" . -}}
{{- end -}}
{{- end -}}

{{/*
    Checks if the expiration date is more 0
*/}}
{{- define "s3.checkExpirationDayValid" -}}
{{- $days := int .days -}}
{{- if le $days 0 -}}
{{- printf "\nValue for expiration days should be greater then 0" | fail -}}
{{- end -}}
{{- end -}}

{{/*
    Check of the right storage class value has been set
*/}}
{{- define "s3.checkStorageClassValid" -}}
{{- $allowedStorageClasses := include "s3.allowedStorageClasses" . }}
{{- if not (has . (splitList "," $allowedStorageClasses)) }}
{{- $error := ( printf "\n Invalid value for transition storage class - [%s]. The following values are supported:[%s]" . $allowedStorageClasses) -}}
{{- fail $error -}}
{{- end -}}
{{- end -}}

{{/*
    Check if the S3 Lifecycle transition config is valid
    - Ensure that the value for days is greater than 0
    - The value for days should be more than 30 for STANDARD_IA and ONEZONE_IA storage classes
    - Check if the value for storage class is valid
*/}}
{{- define "s3.checkTransitionConfigValid" -}}
{{- $days := int .days -}}
{{- $storageClass := .storageClass -}}
{{ include "s3.checkStorageClassValid" $storageClass }}
{{- if le $days 0 -}}
{{- printf "\nValue for transitions days should be greater than 0" | fail -}}
{{- end -}}
{{- if and (lt $days 30) (eq $storageClass "STANDARD_IA") -}}
{{- printf "\nInvalid value for transition days. A minimum of 30 days is required before transitioning to storgae class of type Standard-IA" | fail -}}
{{- end -}}
{{- if and (lt $days 30) (eq $storageClass "ONEZONE_IA") -}}
{{- printf "Invalid value for transition days. A minimum of 30 days is required before transitioning to storgae class of type ONEZONE_IA" | fail -}}
{{- end -}}
{{- end -}}

{{/*
    Configure S3 LifeCycle Rules
    S3 lifecycle rules can be configured in five different ways more infor on that can be found here https://docs.aws.amazon.com/AmazonS3/latest/userguide/object-lifecycle-mgmt.html
    Caveats:
    - ateast one of the properties have to be enabled
    - Abort MultipartUploads
    - Transitions:
        - Current version - transitions
        - Previous version - noncurrent transitions
        - Days cannot be less than 30 for STANDARD_IA  and ONEZONE_IA
        - Allowed values for storage classes: ONEZONE_IA,STANDARD_IA,INTELLIGENT_TIERING,GLACIER,DEEP_ARCHIVE
    - Expirations:
      - Current version - expirations
      - Previous Version - noncurrent expirations
      - Days cannot be smaller than transition days if transition is enabled along with the expiration rule
    - currently Adding date as a property for the lifecycle rule is not enabled to reduce complexity
*/}}
{{- define "s3.lifecycleRules" -}}
{{- $rules := ( required "If .Values.lifecycle.enable has been set as true, You must add atleast one lifecycle rule" .Values.lifecycle.rules ) -}}
[   {{ range $rules  }}
    {{ include "s3.checkLifecyclePropertyValid" . }}
    {
        id = {{ required "You must specify value for lifecycle \"id\" in the lifecycle rule" .id | quote }}
        enabled = {{- .enabled | default false}}
        {{ if .prefix -}}
        prefix = {{- .prefix | quote }}
        {{ end -}}
        {{ if .abortIncompleteMultipartUploadDays -}}
        abort_incomplete_multipart_upload_days = {{- .abortIncompleteMultipartUploadDays }}
        {{ end -}}

        {{ if  .tags -}}
        {{ if .abortIncompleteMultipartUploadDays -}}
        {{ printf "\nTags are not accepted for lifecycle rule created with the property \"abortIncompleteMultipartUploadDays\"" | fail }}
        {{ else }}
        tags = {
            {{ range $key , $value := .tags }}
            {{ $key}} = {{ $value | quote }}
            {{ end }}
        }
        {{ end -}}
        {{ end -}}

        {{ if .expiration -}}
        {{ include "s3.checkExpirationConfigValid" .expiration }}
        expiration = {
            {{ if .expiration.days }}
            days = {{ .expiration.days }}
            {{ end }}
            {{ if .expiration.expiredObjectDeleteMarker }}
            expired_object_delete_marker = {{ .expiration.expiredObjectDeleteMarker }}
            {{ end }}
        }
        {{ end -}}

        {{ if .transitions -}}
        transition = [
            {{ $transitions := .transitions}}
            {{ range $transitions }}
            {{ include "s3.checkTransitionConfigValid" . }}
            {
                days = {{ required "You must specify value for transition days in the lifecycle rule" .days }}
                storage_class = {{ required "You must specify value for storage class to transition to in the lifecycle rule" .storageClass | quote }}
            },
        ]
        {{- end -}}
        {{- end -}}

        {{ if .noncurrentVersionExpiration }}
        {{ include "s3.checkExpirationDayValid" .noncurrentVersionExpiration }}
        noncurrent_version_expiration = {
            days = {{ .noncurrentVersionExpiration.days }}
        }
        {{ end }}

        {{- if .noncurrentVersionTransitions -}}
        noncurrent_version_transition = [
            {{ range .noncurrentVersionTransitions }}
            {
                {{ include "s3.checkTransitionConfigValid" . }}
                days = {{ .days }}
                storage_class = {{ required "You must specify value for storage class to transition to in the lifecycle rule" .storageClass | quote }}
            },
            {{ end }}
        ]
        {{ end }}
    },{{ end }}
]
{{- end -}}

{{- define "s3.corsRules" -}}
{{- $cors_rules := ( required "If .Values.cors.enable has been set as true, You must add atleast one cors rule" .Values.cors.rules ) -}}
[   {{ range $cors_rules  }}
    {
        {{ if .allowed_methods -}}
        allowed_methods = [
            {{- $allowed_methods := .allowed_methods }}
            {{- range $allowed_methods }}
            {{- . | quote }},
            {{- end }}]
        {{- else -}}
        {{- printf "\n\nYou must specify value for cors rule \"allowed_methods\" in the cors rule" | fail -}}
        {{ end }}

        {{ if .allowed_origins -}}
        allowed_origins = [
            {{- $allowed_origins := .allowed_origins }}
            {{- range $allowed_origins }}
            {{- . | quote }},
            {{- end }}]
        {{- else -}}
        {{- printf "\n\nYou must specify value for cors rule \"allowed_origins\" in the cors rule" | fail -}}
        {{ end }}

        {{ if .allowed_headers -}}
        allowed_headers = [
            {{- $allowed_headers := .allowed_headers}}
            {{- range $allowed_headers }}
            {{- . | quote }},
            {{- end }}]
        {{ end }}

        {{ if .expose_headers -}}
        expose_headers = [
            {{- $expose_headers := .expose_headers }}
            {{- range $expose_headers }}
            {{- . | quote }},
            {{- end }}]
        {{ end }}

        {{ if .max_age_seconds -}}
        max_age_seconds = {{- .max_age_seconds }}
        {{ end }}
    },
    {{ end }}
]
{{- end -}}

{{/*
  Define the supported Value for the parameter: .Values.acl
*/}}
{{- define "s3.supportedAcls" -}}
{{- printf "private,public-read,public-read-write,aws-exec-read,authenticated-read,log-delivery-write" }}
{{- end }}

{{/*
  Check if the S3 acl is valid
*/}}
{{- define "s3.bucketAcl" -}}
{{- $acls := include "s3.supportedAcls" . -}}
{{- if (has .Values.acl  (splitList "," $acls)) -}}
{{- .Values.acl -}}
{{ else }}
{{- printf "\n\nInvalid value set for .Values.acl - Only the following acls are supported: [%s]." $acls | fail -}}
{{- end -}}
{{- end -}}

{{/*
   Defines the ouput params for -outputs config map
*/}}
{{- define "s3.outputConfigMapParams" -}}
arnConfigMap: "{{ include "s3-utils.namingPrefix" . }}-arn"
{{ if .Values.existingInfraOverrides.enable }}
bucketName: "{{ required "You must specify the values for '.Values.existingInfraOverrides.bucketName' to be created" .Values.existingInfraOverrides.bucketName }}"
{{ else }}
bucketName: "{{ include "s3.bucketName" . }}"
{{ end }}
{{- end -}}


{{/*
   Define the s3 arn configMap params if existingInfraOverrides flag is set.
   The endpoint configmap is created when deploy-s3.sh is called. Since
   for overrides, we need to populate the same for the existing infrastructure.
*/}}
{{- define "s3.arnConfigMapParams" -}}
{{- if not ( and .Values.existingInfraOverrides.bucketName .Values.existingInfraOverrides.ARN .Values.existingInfraOverrides.BUCKET_REGION ) }}
{{ printf ("\n\nIf you have set \".Values.existingInfraOverrides.enable\" as true then please provide a value for the existing s3 bucketName, ARN, BUCKET_REGION: \".Values.existingInfraOverrides.bucketName\" and \".Values.existingInfraOverrides.ARN\" and \".Values.existingInfraOverrides.BUCKET_REGION\"") | fail }}
{{- end }}
ARN: {{ .Values.existingInfraOverrides.ARN }}
BUCKET_DOMAIN_NAME: {{ .Values.existingInfraOverrides.bucketName }}.s3.amazonaws.com
BUCKET_REGIONAL_DOMAIN_NAME: {{ .Values.existingInfraOverrides.bucketName }}.s3.{{ .Values.existingInfraOverrides.BUCKET_REGION }}.amazonaws.com
{{- if .Values.existingInfraOverrides.BucketLifecyclePolicy }}
BucketLifecyclePolicy: {{ .Values.existingInfraOverrides.BucketLifecyclePolicy }}
{{- end }}
{{- if .Values.existingInfraOverrides.ReadOnlyPolicy }}
ReadOnlyPolicy: {{ .Values.existingInfraOverrides.ReadOnlyPolicy }}
{{- end }}
{{- if .Values.existingInfraOverrides.ReadWritePolicy }}
ReadWritePolicy: {{ .Values.existingInfraOverrides.ReadWritePolicy }}
{{- end }}
{{- end -}}

